#!/usr/bin/env python3

# Copyright (C) 2010-2020, Josef Hahn
#
# This file is part of Ginger.
#
# Ginger is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ginger is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import subprocess
import hashlib
import time


def main():

    cmd = os.path.basename(sys.argv[0])

    if len(sys.argv) == 1:

        print("""Usage:

        ${cmd} [command] ... ...

    For more information, please read the ginger documentation.

        """.format(**locals()))
        sys.exit(1)

    else:

        command = sys.argv[1]

        _install_dir = os.path.abspath(__file__)
        install_dir = None
        while install_dir is None:
            for cnd in [_install_dir + "", _install_dir + "/ginger"]:
                if os.path.isfile(cnd + "/main/core.py"):
                    install_dir = _install_dir
                    break
            if install_dir:
                break
            _ninstalldir = os.path.dirname(_install_dir)
            if _install_dir == _ninstalldir:
                raise Exception("installation directory not found")
            _install_dir = _ninstalldir
        install_dir = os.path.realpath(install_dir)

        inner_install_dir = install_dir
        if os.path.isdir(install_dir + "/ginger"):
            inner_install_dir = os.path.realpath(install_dir + "/ginger")

        # detect python executable
        pythoncmd = "python" + str(sys.version_info[0])
        try:
            subprocess.call([pythoncmd, "-V"])
        except OSError:
            pythoncmd = "python"

        if os.path.exists(install_dir + "/manage.py"):
            managecmd = install_dir + "/manage.py"
        else:
            managecmd = install_dir + "/_meta/manage.py"

        # begin command implementations

        if command == "setup" or command == "init" or command == "update":

            os.chdir(inner_install_dir)

            if command == "setup":

                runtime_dir = os.path.abspath(os.path.expandvars(sys.argv[2])).replace("\\", "/")

                if not os.path.isdir(runtime_dir):
                    os.makedirs(runtime_dir)

                secretkey = hashlib.md5(str(time.time()).encode("ascii")).hexdigest()

                r = "}"
                l = "{"

                settingslocaldir = inner_install_dir + "/settings_local.py"

                with open(settingslocaldir, "w") as f:
                    f.write("""
DEBUG = True  # required also for static files delivery in dev server (for any reason)
SECRET_KEY = '{secretkey}'
STATIC_ROOT = "{runtime_dir}/static"

import os
if "GINGER_DB_ENGINE" in os.environ:  # mostly for container images
    _db = {l}{r}
    for k in os.environ:
        if k.startswith("GINGER_DB_"):
            _db[k[10:]] = os.environ[k]
    DATABASES = {l} 'default': _db {r}
else:
    DATABASES = {l}
        'default': {l}
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': '{runtime_dir}/ginger-db-v1',
        {r}
    {r}
""".format(**locals()))

            if command == "setup" or command == "init":

                subprocess.call([pythoncmd, managecmd, "makemigrations", "main"])
                subprocess.call([pythoncmd, managecmd, "collectstatic", "--noinput"])

            subprocess.call([pythoncmd, managecmd, "makemigrations"])
            subprocess.call([pythoncmd, managecmd, "migrate"])

        elif command == "runserver":

            os.chdir(install_dir)
            subprocess.call([pythoncmd, managecmd, "runserver"])

        else:

            print("unknown command: " + command)
            sys.exit(1)


if __name__ == "__main__":
    main()
