# Copyright (C) 2010-2020, Josef Hahn
#
# This file is part of Ginger.
#
# Ginger is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ginger is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

import json
import subprocess
import django.conf
import django.contrib.auth
import django.contrib.auth.models
import django.core.serializers
import django.db
import django.http
import django.template
import django.utils.safestring

import ginger.main.messagefilter
import ginger.main.models


json_serializer = django.core.serializers.get_serializer("json")()


def init_ginger(request):
    if not django.conf.settings.EXTERNAL_AUTH_HELPER:
        newadmin = django.contrib.auth.models.User.objects.create_user('admin', 'admin@localhost', 'admin')
        newadmin.is_superuser = True
        newadmin.is_staff = True
        newadmin.save()


def _dumpmessagedata(entries, user):
    return {"elements": json_serializer.serialize(entries),
            "tags": json_serializer.serialize(ginger.main.models.Tag.objects.filter(owner=user))}


def list_all_messages(request):
    try:
        idthresh = int(request.GET["lasttimestamp"])
    except Exception:
        idthresh = -1
    if request.user.is_authenticated():
        elements = ginger.main.models.NewsMessage.objects.filter(
            deleted=False, owner=request.user, fetchedAt__gt=idthresh)
    if "filter" in request.GET and request.GET["filter"] != "":
        f = ginger.main.messagefilter.filterbyfilterstring(request.GET["filter"])
        elements = f.applyfilter(elements, request.user)
    elements = sorted(elements, key=lambda element: element.created)
    return django.http.HttpResponse(json.dumps(_dumpmessagedata(elements, request.user)))


def store_filter(request):
    ginger.main.models.UserConfiguration.set(
        request.user, "filter:" + request.POST["name"], json.dumps(ginger.main.messagefilter.filterbyfilterstring(
                              request.POST["v"]).tonativerepresentation()))
    return django.http.HttpResponse("1")


def list_filter(request):
    filters = []
    for k in ginger.main.models.UserConfiguration.list(request.user):
        if k.startswith("filter:"):
            filters.append((k[7:], ginger.main.models.UserConfiguration.get(request.user, k, "")))
    return django.http.HttpResponse(json.dumps({"filter": filters}))


def remove_filter(request):
    ginger.main.models.UserConfiguration.remove(request.user, "filter:" + request.POST["name"])
    return django.http.HttpResponse("1")


def store_default_filter(request):
    f = json.dumps(ginger.main.messagefilter.filterbyfilterstring(
        request.POST["filterstring"]).tonativerepresentation())
    ginger.main.models.UserConfiguration.set(request.user, "defaultFilter", f)
    return django.http.HttpResponse("1")


def mark_as_seen(request):
    elemid = int(request.POST["id"])
    m = ginger.main.models.NewsMessage.objects.get(pk=elemid)
    if m.owner == request.user:
        m.seen = True
        m.save()
    return django.http.HttpResponse("1")


def delete_message(request):
    elemid = int(request.POST["id"])
    m = ginger.main.models.NewsMessage.objects.get(pk=elemid)
    if m.owner == request.user:
        m.deletemessage()
    return django.http.HttpResponse("1")


def set_tags(request):
    elemid = int(request.POST["id"])
    m = ginger.main.models.NewsMessage.objects.get(pk=elemid)
    if m.owner == request.user:
        m.settags([request.POST[k] for k in request.POST.keys() if k.startswith("tag")])
    return django.http.HttpResponse(
        json.dumps(_dumpmessagedata(ginger.main.models.NewsMessage.objects.filter(pk=elemid), request.user)))


def set_tag(request):
    elemid = int(request.POST["id"])
    tag = request.POST["tag"]
    m = ginger.main.models.NewsMessage.objects.get(pk=elemid)
    if m.owner == request.user:
        m.addtag(tag)
    return django.http.HttpResponse(
        json.dumps(_dumpmessagedata(ginger.main.models.NewsMessage.objects.filter(pk=elemid), request.user)))


def unset_tag(request):
    elemid = int(request.POST["id"])
    tag = request.POST["tag"]
    m = ginger.main.models.NewsMessage.objects.get(pk=elemid)
    if m.owner == request.user:
        m.removetag(tag)
    return django.http.HttpResponse(
        json.dumps(_dumpmessagedata(ginger.main.models.NewsMessage.objects.filter(pk=elemid), request.user)))


def do_login(request):
    username = request.POST["username"]
    password = request.POST["password"]
    # external auth?
    if not django.conf.settings.EXTERNAL_AUTH_HELPER:
        u = django.contrib.auth.authenticate(username=username, password=password)
    else:
        extp = subprocess.Popen([django.conf.settings.EXTERNAL_AUTH_HELPER, ], stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        username = extp.communicate((username + "\n" + password + "\n").encode())[0].decode().strip()
        u = None
        if len(username) > 0:
            m = django.contrib.auth.models.User.objects.filter(username=username)
            if len(m) == 1:
                u = django.contrib.auth.authenticate(username=username, password=password)
                if u is None:
                    m[0].set_password(password)
                    m[0].save()
                    u = django.contrib.auth.authenticate(username=username, password=password)
            elif len(m) == 0:
                u = django.contrib.auth.models.User.objects.create_user(username, '', password)
                u.save()
                u = django.contrib.auth.authenticate(username=username, password=password)
    if u is not None:
        if u.is_active:
            django.contrib.auth.login(request, u)
            return django.http.HttpResponse(get_usersession_json(u))
    return django.http.HttpResponse(json.dumps({'loginfailed': True}))


def do_logout(request):
    django.contrib.auth.logout(request)
    return django.http.HttpResponse("1")


def test_if_ginger_initialized(request):
    s = False
    if (len(django.contrib.auth.models.User.objects.all()) +
            len(ginger.main.models.Feed.objects.all()) +
            len(ginger.main.models.NewsMessage.objects.all())) == 0 and \
            (not django.conf.settings.EXTERNAL_AUTH_HELPER):
        init_ginger(request)
        s = True
    return s


def get_usersession_json(user):
    if user.is_authenticated():
        return json.dumps({
            'user': user.username if user.is_authenticated() else None,
            'listdirection': -1 if ginger.main.models.UserConfiguration.get(user, "reverseSortOrder", "0") == "1" else 1,
            'defaultfilter': ginger.main.models.UserConfiguration.get(user, "defaultFilter", "") or None,
            'powermode': ginger.main.models.UserConfiguration.get(user, "powermode", "0") == "1",
            'superuser': user.is_superuser,
            'onuiinitialized': ginger.main.models.UserConfiguration.get(user, "onuiinitialized", ""),
        })
    else:
        return json.dumps(None)


def return_index_html(request):
    try:
        freshly = test_if_ginger_initialized(request)
    except django.db.OperationalError as e:
        raise Exception("Your database seems to be wrongly configured or not initialized! Details: " + str(e))
    t = django.template.loader.get_template("index.html")
    if django.VERSION >= (1, 8):  # FIXME compatibility with old version
        tt = dict
    else:
        tt = django.template.Context
    return django.http.HttpResponse(t.render(tt({
                                    'subsite': django.conf.settings.SUB_SITE,
                                    'sessioncfg': django.utils.safestring.mark_safe(json.dumps({
                                        'subsite': django.conf.settings.SUB_SITE,
                                        'freshuser': freshly,
                                    })),
                                    'usercfg': django.utils.safestring.mark_safe(get_usersession_json(request.user))})))


def listconfigurationvalues(request):
    return django.http.HttpResponse(json.dumps({"list": ginger.main.models.UserConfiguration.list(request.user)}))


def getconfigurationvalue(request):
    key = request.GET["key"]
    return django.http.HttpResponse(json.dumps({"value": ginger.main.models.UserConfiguration.get(request.user, key)}))


def setconfigurationvalue(request):
    key = request.GET["key"]
    value = request.POST["value"]
    ginger.main.models.UserConfiguration.set(request.user, key, value)
    return django.http.HttpResponse("1")
