# Copyright (C) 2010-2020, Josef Hahn
#
# This file is part of Ginger.
#
# Ginger is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ginger is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import feedparser
import threading
import time
import traceback
import django.http
import django.db.transaction

import ginger.main.models
import ginger.main.htmlcleaner


def garbage_collector(feed, seen_msgs):
    res = ""
    for msg in ginger.main.models.NewsMessage.objects.filter(deleted=True, feed=feed):
        if not (msg.guid in seen_msgs):
            res += "Deleting '%s'...\n" % (msg.guid,)
            msg.delete()
    return res


def crawl(request):
    response = ""
    # lnow is an abstract integer that increases by each fetched message and
    # thereby provides an unique value along all message.
    # will be stored in the messages, so you can sort or filter by it
    lnow = (int(time.time()) - 1000000000) * 1000
    for feed in ginger.main.models.Feed.objects.all():
        try:
            try:
                atomic = django.db.transaction.atomic
            except AttributeError:
                atomic = threading.Lock  # last (poor) resort for aged djangos
            with atomic():
                if datetime.datetime.now() - feed.lastFetched >= datetime.timedelta(minutes=feed.updateInterval):
                    feed.lastFetched = datetime.datetime.now()
                    feed.save()
                    fetchfeed = True
                else:
                    fetchfeed = False
            if fetchfeed:
                response += "Processing feed '%s'...\n" % (feed.name)
                _seen_msgs = []  # we need this for collecting garbage in message storage
                if feed.enabled:
                    response += "Crawling...\n"
                else:
                    response += garbage_collector(feed, _seen_msgs)
                    response += "Skipping disabled.\n"
                    continue
                tempfeed = feedparser.parse(feed.url)
                for feeditem in tempfeed['items']:
                    # check if there is a timstamp and its format
                    if 'updated_parsed' in feeditem:
                        timestamp = feeditem['updated_parsed']
                        itemtimestamp = datetime.datetime(*timestamp[:6])
                    elif 'published_parsed' in feeditem:
                        timestamp = feeditem['published_parsed']
                        itemtimestamp = datetime.datetime(*timestamp[:6])
                    elif 'date_parsed' in feeditem:
                        timestamp = feeditem['date_parsed']
                        itemtimestamp = datetime.datetime(*timestamp[:6])
                    else:
                        itemtimestamp = datetime.datetime.now()
                    # get URL
                    itemurl = feeditem['link']
                    # get title
                    itemtitle = feeditem['title']
                    resttitle = ''
                    if len(itemtitle) > 140:
                        # look form blank char at position > 140
                        cut = itemtitle.find(" ", 140)
                        if cut == -1:
                            cut = 140
                        temptitle = itemtitle[0:cut] + ' ...'
                        resttitle = '... ' + itemtitle[cut:]
                        itemtitle = temptitle
                    # check if there is a summary
                    if 'summary' in feeditem:
                        itemsummary = feeditem['summary']
                    else:
                        itemsummary = ''
                    # throw away most html stuff
                    itemsummary = ginger.main.htmlcleaner.cleanhtml(itemsummary, "_blank")
                    if resttitle:
                        itemsummary = resttitle + '\n' + itemsummary
                    # get item id
                    unique = False
                    if 'guid' in feeditem:
                        itemguid = feeditem['guid']
                        unique = True
                    elif 'id' in feeditem:
                        itemguid = feeditem['id']
                    else:
                        itemguid = itemurl
                    _seen_msgs.append(itemguid)
                    # check if this is a double entry
                    filterargs = {} if unique else {feed: feed}
                    l = ginger.main.models.NewsMessage.objects.filter(guid=itemguid, **filterargs)
                    if len(l) == 0:
                        # write to db
                        m = ginger.main.models.NewsMessage(title=itemtitle, summary=itemsummary, feed=feed, url=itemurl,
                                                           guid=itemguid, owner=feed.owner, created=itemtimestamp,
                                                           fetchedAt=lnow)
                        lnow += 1
                        ginger.main.models.NewsMessage.addmessage(m)
                        m.addtags(feed.correspondingTags.all())
                        response += "Adding '%s'...\n" % (itemguid,)
                response += garbage_collector(feed, _seen_msgs)
        except Exception:
            response += traceback.format_exc() + "\n"
    return django.http.HttpResponse(response.replace("\n", "<br/>"))
